@@ -90,7 +90,7 @@ class ALE_Token
 	
 	/* String correspounding to the token, defined at the end of token
 	   reading. */
-	var $string;
+	var $text;
 	
 	/* Properties of the token, used by transition actions to store token
 	   related data. */
@@ -99,12 +99,25 @@ class ALE_Token
 	/*
 	 * Creates a new token.
 	 */
-	function ALE_Token($tokenType = null, $string = '', $properties = array())
+	function ALE_Token($tokenType = null, $text = '', $properties = array())
 	{
 		$this->tokenType = $tokenType;
-		$this->string = $string;
+		$this->text = $text;
 		$this->properties = $properties;
 	}
+	
+	/* Relevant token text, according to token type and properties (for
+	   serialization). */
+	function &relevantText()
+	{
+		return $this->tokenType->relevantText($this);
+	}
+	
+	/* Relevant token properties subset (for serialization). */
+	function &relevantProperties()
+	{
+		return $this->tokenType->relevantProperties($this);
+	}
 }
 
 /*
@@ -113,6 +126,19 @@ class ALE_Token
 class ALE_TokenType
 {
 	var $name;
+	
+	/* Relevant token text, according to token type and properties (for
+	   serialization). */
+	function &relevantText(&$token)
+	{
+		return $token->text;
+	}
+	
+	/* Relevant token properties subset (for serialization). */
+	function &relevantProperties(&$token)
+	{
+		return $token->properties;
+	}
 }
 
 /*
