@@ -45,12 +45,83 @@ class WS_ProtocolsTransition extends ALE_Transition
 		$this->actions =& $actions;
 	}
 	
-	
 	function isApplicableTo(&$context)
 	{
 		global $WS_protocolsPattern;
-		return preg_match("#^($WS_protocolsPattern)$#",
-			$context->currentString.$context->currentChar);
+		return array(
+			preg_match(
+				"#^($WS_protocolsPattern)$#",
+				$context->currentString.$context->currentChar
+			),
+		$this->consumption);
+	}
+}
+
+/*
+ * "cancel" transition, used to cancel characters consumption when detecting a
+ * bad recognition.
+ */
+class WS_CancelCharTransition extends ALE_Transition
+{
+	var $keep;
+	
+	/*
+	 * Create a "cancel" transition, applicable when current character
+	 * belongs to a given class.
+	 * $charClass : character class ("preg" regex form, '\w' or '[\d\.-]',
+	 * for example).
+	 * $targetState : target state name.
+	 * $keep : number of character keep consumed when the transition is
+	 * applyed.
+	 * $actions : sequence of actions to execute juste before cancellation.
+	 */
+	function WS_CancelCharTransition($charClass, $targetState, $keep = 0,
+		$actions = array())
+	{
+		$this->charClass = preg_replace(',#,', '\#', $charClass);
+		$this->targetState = $targetState;
+		$this->keep = $keep;
+		$this->actions =& $actions;
+	}
+	
+	function isApplicableTo(&$context)
+	{
+		return array(
+			preg_match('#'.$this->charClass.'#', $context->currentChar),
+			$this->keep - strlen($context->currentString)
+		);
+	}
+}
+
+/*
+ * "cancel" transition, used to cancel characters consumption when reaching
+ * data end too early.
+ */
+class WS_CancelEndTransition extends ALE_Transition
+{
+	var $keep;
+	
+	/*
+	 * Create a "cancel" transition, applicable when the end of data is
+	 * reached.
+	 * $targetState : target state name.
+	 * $keep : number of character keep consumed when the transition is
+	 * applyed.
+	 * $actions : sequence of actions to execute juste before cancellation.
+	 */
+	function WS_CancelEndTransition($targetState, $keep = 0, $actions = array())
+	{
+		$this->targetState = $targetState;
+		$this->keep = $keep;
+		$this->actions =& $actions;
+	}
+	
+	function isApplicableTo(&$context)
+	{
+		return array(
+			$context->endOfData,
+			$this->keep - strlen($context->currentString)
+		);
 	}
 }
 
@@ -86,19 +157,19 @@ $WS_states = array(
 		)
 	),
 	'smiley0' => state(
-		endOfData('text'),
+		new WS_CancelEndTransition('text', 2),
 		other('smiley0', CONS),
 		array(
-			char('[\r\n<\[]', 'text', NOCONS),
+			new WS_CancelCharTransition('[\r\n<\[]', 'text', 2),
 			char('\]', 'smiley', CONS, array('WS_A_smileyName'))
 		)
 	),
 	'smiley' => term($WS_T_Smiley),
 	'wiki0' => state(
-		endOfData('text'),
+		new WS_CancelEndTransition('text', 1),
 		other('wiki0', CONS),
 		array(
-			char('[\r\n<\[]', 'text', NOCONS),
+			new WS_CancelCharTransition('[\r\n<\[]', 'text', 1),
 			char('\]', 'wiki', CONS, array('WS_A_wikiKeyword')),
 			char('@', 'wikiname', CONS, array('WS_A_wikiKeyword'))
 		)
