@@ -52,25 +52,32 @@ class WS_V_PrinterXml extends WS_Visitor
 		$this->out = WS_OUT_XML;
 	}
 	
+	function visitToken_text(&$token)
+	{
+		return htmlspecialchars($token->text);
+	}
+	
 	function visitToken_blank(&$token)
 	{
 		return ' ';
 	}
 	
-	function visitToken_smiley(&$token)
+	function visitToken_url(&$token)
 	{
-		global $WS_smileyWiki;
-		return ($WS_smileyWiki == '')
-			? htmlspecialchars($token->string)
-			: "<a href=\"$WS_smileyWiki%3a"
-				.urlspecialchars($token->properties['name'])."\">"
-				.htmlspecialchars($token->string)."</a>";
+		$url = isset($token->properties['url'])
+			? $token->properties['url']
+			: urlspecialchars($token->text);
+		$title = isset($token->properties['title'])
+			? $token->properties['title']
+			: '[url]';
+		return "<b><a href=\"$url\">$title</a></b>";
 	}
 	
 	function visitToken_wiki(&$token)
 	{
 		global $WS_defaultWiki, $WS_wikis;
-		if ($WS_defaultWiki == "") return htmlspecialchars($token->string);
+		$text = $token->tokenType->xmlText($token);
+		if ($WS_defaultWiki == "") return $text;
 		$baseUrl = isset($token->properties['site'])
 			? "http://".$token->properties['site']
 			: (isset($token->properties['name'])
@@ -78,8 +85,17 @@ class WS_V_PrinterXml extends WS_Visitor
 				: $WS_defaultWiki
 			);
 		return "<a href=\"$baseUrl"
-			.urlspecialchars($token->properties['keyword'])."\">"
-			.htmlspecialchars($token->string)."</a>";
+			.urlspecialchars($token->properties['keyword'])."\">$text</a>";
+	}
+	
+	function visitToken_smiley(&$token)
+	{
+		global $WS_smileyWiki;
+		$text = $token->tokenType->xmlText($token);
+		return ($WS_smileyWiki == '')
+			? $text
+			: "<a href=\"$WS_smileyWiki%3a"
+				.urlspecialchars($token->properties['name'])."\">$text</a>";
 	}
 	
 	function visitToken_begintag(&$token)
@@ -104,25 +120,9 @@ class WS_V_PrinterXml extends WS_Visitor
 			: "&lt;/$name&gt;";
 	}
 	
-	function visitToken_text(&$token)
-	{
-		return htmlspecialchars($token->string);
-	}
-	
-	function visitToken_url(&$token)
-	{
-		$url = isset($token->properties['url'])
-			? $token->properties['url']
-			: urlspecialchars($token->string);
-		$title = isset($token->properties['title'])
-			? $token->properties['title']
-			: '[url]';
-		return "<b><a href=\"$url\">$title</a></b>";
-	}
-	
 	function visitToken_clock(&$token)
 	{
-		return $token->string;
+		return $token->text;
 	}
 }
 
@@ -138,10 +138,38 @@ class WS_V_PrinterHtml extends WS_V_PrinterXml
 	{
 		$this->out = WS_OUT_HTML;
 	}
+	
+	function visitToken_url(&$token)
+	{
+		$url = isset($token->properties['url'])
+			? $token->properties['url']
+			: urlspecialchars($token->text);
+		$title = isset($token->properties['title'])
+			? $token->properties['title']
+			: '[url]';
+		return "<b><a href=\"$url\" title=\"$title\">$title</a></b>";
+	}
+	
+	function visitToken_wiki(&$token)
+	{
+		global $WS_defaultWiki, $WS_wikis;
+		$text = $token->tokenType->xmlText($token);
+		if ($WS_defaultWiki == "") return $text;
+		$baseUrl = isset($token->properties['site'])
+			? "http://".$token->properties['site']
+			: (isset($token->properties['name'])
+				? $WS_wikis[$token->properties['name']]
+				: $WS_defaultWiki
+			);
+		return "<a href=\"$baseUrl"
+			.urlspecialchars($token->properties['keyword'])."\""
+			." title=\"$text\">$text</a>";
+	}
+	
 	function visitToken_smiley(&$token)
 	{
 		global $WS_smileyUrl, $WS_smileyWiki;
-		$text = htmlspecialchars($token->string);
+		$text = $token->tokenType->xmlText($token);
 		$smiley = ($WS_smileyUrl == "")
 			? $text
 			: preg_replace(',%s,', rawurlencode($token->properties['name']),
@@ -150,41 +178,14 @@ class WS_V_PrinterHtml extends WS_V_PrinterXml
 		return ($WS_smileyWiki == "")
 			? $smiley
 			: "<a href=\"$WS_smileyWiki%3a"
-				.urlspecialchars($token->properties['name'])."\" title=\""
-				.htmlspecialchars($token->string)."\">$smiley</a>";
-	}
-	
-	function visitToken_wiki(&$token)
-	{
-		global $WS_defaultWiki, $WS_wikis;
-		if ($WS_defaultWiki == "") return htmlspecialchars($token->string);
-		$baseUrl = isset($token->properties['site'])
-			? "http://".$token->properties['site']
-			: (isset($token->properties['name'])
-				? $WS_wikis[$token->properties['name']]
-				: $WS_defaultWiki
-			);
-		return "<a href=\"$baseUrl"
-			.urlspecialchars($token->properties['keyword'])."\" title=\""
-			.htmlspecialchars($token->string)."\">"
-			.htmlspecialchars($token->string)."</a>";
-	}
-	
-	function visitToken_url(&$token)
-	{
-		$url = isset($token->properties['url'])
-			? $token->properties['url']
-			: urlspecialchars($token->string);
-		$title = isset($token->properties['title'])
-			? $token->properties['title']
-			: '[url]';
-		return "<b><a href=\"$url\" title=\"$title\">$title</a></b>";
+				.urlspecialchars($token->properties['name'])."\""
+				." title=\"$text\">$smiley</a>";
 	}
 	
 	function visitToken_clock(&$token) /* see rfcc40 */
 	{
 		global $WS_boards;
-		if (!$token->properties['valid']) return $token->string;
+		if (!$token->properties['valid']) return $token->text;
 		
 		$board = '';
 		$url = '';
@@ -215,7 +216,7 @@ class WS_V_PrinterHtml extends WS_V_PrinterXml
 		return "<a class='_$clock ".join(' ', $classes)."' "
 			."onmouseover=\"highlight('$clock$board')\" "
 			."onmouseout=\"unhighlight()\" "
-			."href=\"$url#$clock\">$token->string</a>";
+			."href=\"$url#$clock\">$token->text</a>";
 	}
 }
 
