@@ -230,15 +230,15 @@ function WS_checkLength(&$tokens, $postMaxLength, $wordMaxLength)
 	$postLength = 0;
 	$wordLength = 0;
 	foreach($tokens as $token) {
-		list($length, $isText) = $token->tokenType->getLength($token);
+		$length = strlen($token->relevantText());
 		$postLength += $length;
-		if ($isText) $wordLength += $length;
-		else $wordLength = 0;
+		if ($token->tokenType->isSpace($token)) $wordLength = 0;
+		else $wordLength += $length;
 /*
 		print("<p>");
 		print("<tt>".$token->tokenType->name."</tt> : ");
-		print("<i>".htmlspecialchars($token->string)."</i> ; ");
-		print("\$postLength = $postLength ; \$wordLength = $wordLength");
+		print("<i>".htmlspecialchars($token->text)."</i> ; ");
+		//print("\$postLength = $postLength ; \$wordLength = $wordLength");
 		print("</p>\n");
 */
 		if ($postLength > $postMaxLength) return WS_LENGTH_NOK_POST;
@@ -252,7 +252,7 @@ function WS_checkLength(&$tokens, $postMaxLength, $wordMaxLength)
  * $tokens : tokens list.
  * $out : output format.
  */
-function WS_getOutput(&$tokens, $out)
+function &WS_getOutput(&$tokens, $out)
 {
 	global $WS_Visitors;
 	$output = '';
@@ -265,25 +265,27 @@ function WS_getOutput(&$tokens, $out)
 /*
  * Serialize a token (list)* according to rfcc39.
  */
-function WS_serialize(&$tokens)
+function &WS_serialize(&$tokens)
 {
-	$serializer = new ALE_Serializer();
+	global $WS_InitSymbTab;
+	$serializer = new ALE_Serializer($WS_InitSymbTab);
 	return $serializer->serialize($tokens);
 }
 
 /*
  * Deserialize a token (list)* according to rfcc39.
  */
-function WS_unserialize(&$tokens)
+function &WS_unserialize(&$tokens)
 {
-	global $WS_TokenTypes;
-	$unserializer = new ALE_Unserializer($WS_TokenTypes, $tokens);
+	global $WS_TokenTypes, $WS_InitSymbTab;
+	$unserializer = new ALE_Unserializer($WS_TokenTypes, $tokens,
+		$WS_InitSymbTab);
 	return $unserializer->unserialize();
 }
 
 /*
- * Returns an equivalent token list where successive text or blank tokens were
- * merged (each blank token is replaced by a single space character).
+ * Returns an equivalent token list where successive textual tokens were
+ * merged (according to their relevant text).
  */
 function &WS_mergeTextTokens(&$tokens)
 {
@@ -292,30 +294,24 @@ function &WS_mergeTextTokens(&$tokens)
 	$newTokens = array();
 	$last = 0;
 	
-	if ($tokens[0]->tokenType == $WS_T_Blank) {
-		$newTokens[0] = new ALE_Token($WS_T_Text, ' ');
-	} else if ($tokens[0]->tokenType == $WS_T_Text) {
-		$newTokens[0] = new ALE_Token($WS_T_Text, $tokens[0]->string);
+	if ($tokens[0]->tokenType->isText($tokens[0])) {
+		$newTokens[0] = new ALE_Token($WS_T_Text, $tokens[0]->relevantText());
 	} else {
 		$newTokens[0] = $tokens[0];
 	}
 	
 	for ($i = 1; $i < count($tokens); $i++) {
 		if ($newTokens[$last]->tokenType == $WS_T_Text) {
-			if ($tokens[$i]->tokenType == $WS_T_Blank) {
-				$newTokens[$last]->string .= ' ';
-			} else if ($tokens[$i]->tokenType == $WS_T_Text) {
-				$newTokens[$last]->string .= $tokens[$i]->string;
+			if ($tokens[$i]->tokenType->isText($tokens[$i])) {
+				$newTokens[$last]->text .= $tokens[$i]->relevantText();
 			} else {
 				array_push($newTokens, $tokens[$i]);
 				$last++;
 			}
 		} else {
-			if ($tokens[$i]->tokenType == $WS_T_Blank) {
-				array_push($newTokens, new ALE_Token($WS_T_Text, ' '));
-			} else if ($tokens[$i]->tokenType == $WS_T_Text) {
+			if ($tokens[$i]->tokenType->isText($tokens[$i])) {
 				array_push($newTokens,
-					new ALE_Token($WS_T_Text, $tokens[$i]->string));
+					new ALE_Token($WS_T_Text, $tokens[$i]->relevantText()));
 			} else {
 				array_push($newTokens, $tokens[$i]);
 			}
