@@ -185,7 +185,7 @@ WS_addAllowedTag('m');
 WS_addTagReplacement('m', '====> <b>Moment ', '</b> <====');
 //WS_addTagReplacement('u', '<span style="text-decoration: underline">', '</span>', WS_OUT_HTML);
 //WS_addTagReplacement('s', '<span style="text-decoration: line-through">', '</span>', WS_OUT_HTML);
-WS_setAllowedProtocols(array('http://', 'https://', 'ftp://', 'mailto:', 'ipot://', 'ed2k://', 'pro#to://'));
+WS_setAllowedProtocols(array('http://', 'https://', 'ftp://', 'mailto:', 'ipot://', 'ed2k://', 'rsync://'));
 
 function bugNumber(&$context, &$properties)
 {
@@ -248,11 +248,11 @@ print("</p>");
 print("<hr />\n<h4>wml code (encoded)</h4>\n<p>");
 print(htmlspecialchars(preg_replace(',\$,', '$$', WS_getOutput($tokens, WS_OUT_WML))));
 print("</p>");
-
+/*
 print("<hr />\n<h4>serialized tokens (encoded)</h4>\n<p>");
 print(htmlspecialchars(WS_serialize($tokens)));
 print("</p>");
-
+*/
 print("<hr />\n<h4>serialized merged tokens (encoded)</h4>\n<p>");
 print(htmlspecialchars(WS_serialize(WS_mergeTextTokens($tokens))));
 print("</p>");
